from typing import List, Tuple
import pandas as pd
from talp_pages.io.run_folders import RunFolder
from talp_pages.io.dataframe_handling import get_dfs
from dataclasses import dataclass
from datetime import datetime
@dataclass
class TimeSeriesAnalysisArguments:
    run_folder: RunFolder

@dataclass
class TimeSeriesAnalysisResult:
    ressource_configuration: str
    dataframe: pd.DataFrame
    latest_change: Tuple[datetime,str] # Timestamp and git commit if available

class TimeSeriesAnalysis:
    def __init__(self, arguments: TimeSeriesAnalysisArguments) -> None:
        if not arguments:
            raise ValueError("No arguments given to TimeSeriesAnalysis")
        self.run_folder = arguments.run_folder
        # get df from run_folders
        self.df = get_dfs([self.run_folder], as_list=False) # we only have one


    def get_results(self,ressource_configurations:List[str]) -> List[TimeSeriesAnalysisResult]:
        # Returns a result for every region
        results = []
        for ressource_configuration in ressource_configurations:
            dataframe = self.df[self.df['ressourceLabel'] == ressource_configuration]
            dataframe = dataframe.sort_values(by=["timestamp","elapsedTime"], ascending=[False,False])
            row_of_newest_timestamp = dataframe.loc[dataframe['timestamp'].idxmax()]
            date = pd.to_datetime(row_of_newest_timestamp['timestamp'])

            git_commit = str(row_of_newest_timestamp['gitCommitShort'])
         
            latest_change = (date,git_commit)
           
            results.append(TimeSeriesAnalysisResult(ressource_configuration=ressource_configuration,
                                                    dataframe=dataframe,
                                                    latest_change = latest_change,
                                                    )
            )

        return results


from typing import List, Union
import pandas as pd
from pathlib import Path
import json
from datetime import datetime
from talp_pages.io.run_folders import RunFolder
from talp_pages.common import TALP_JSON_TIMESTAMP_KEY,TALP_JSON_METADATA_KEY,ExecutionMode, TALP_VERSION_KEY, TALP_IMPLICIT_REGION_NAME,TALP_JSON_POP_METRICS_KEY,TALP_RESOURCES_KEY

import logging


def detect_execution_mode(df) -> ExecutionMode:
    # First check if for all its 1
    total_threads = df['totalNumCpus'] - df['totalNumMpiRanks']
    if df['totalNumCpus'].sum() == len(df['totalNumCpus']):
        # Only 1 totalNumCpu per entry -> all entries are serial
        return ExecutionMode.SERIAL
    elif df['totalNumMpiRanks'].sum() == 0:
        # Not all runs are serial, but no MPI Ranks: 
        return ExecutionMode.OPENMP
    elif total_threads.sum() == 0:
        # We have MPI Ranks, but no additional CPUs #TODO Think of LeWI
        return ExecutionMode.MPI
    else: 
        return ExecutionMode.HYBRID


def __load_talp_json_3_5(json_input,json_path):

    
    # we do this a bit ugly for now and reorganize the json a bit: 
    for region in json_input[TALP_JSON_POP_METRICS_KEY].keys():
        json_input[TALP_JSON_POP_METRICS_KEY][region]['regionName'] = region   

    df = pd.DataFrame(json_input[TALP_JSON_POP_METRICS_KEY].values())
    try:
        df['runTimestamp'] = pd.to_datetime(json_input[TALP_JSON_TIMESTAMP_KEY])
    except KeyError:
        df['runTimestamp']=pd.to_datetime(datetime.now())
        logging.critical(f"Could not read {TALP_JSON_TIMESTAMP_KEY} key in {json_path}. Using current time instead")
    try: 
        metadata = json_input[TALP_JSON_METADATA_KEY]
    except KeyError:
        metadata = {}
    df['metadata'] = json.dumps(metadata)
    
    if metadata:
        df['gitCommitShort'] = metadata["gitCommit"][:7] # Git Short shas have 7 chars
        try: 
            df['gitTimestamp'] = pd.to_datetime(metadata["gitTimestamp"])
            df['timestamp'] = pd.to_datetime(metadata["gitTimestamp"])
        except KeyError:
            df['timestamp'] = df['runTimestamp']
    else:
        df['timestamp'] = df['runTimestamp']
        df['gitCommitShort'] = "" 
    

    try:
        resources = json_input[TALP_RESOURCES_KEY]
    except KeyError:
        resources = {
            "numCpus": 0,
            "numMpiRanks": 0
        }
    
    num_cpus = resources['numCpus']
    num_mpi_ranks = resources['numMpiRanks']
    df['totalNumCpus']=num_cpus
    df['totalNumMpiRanks']=num_mpi_ranks

    
    df['dlbVersion'] = "3.5"
    df['jsonFile'] = str(json_path)
    df['implicitRegionName'] = TALP_IMPLICIT_REGION_NAME

    # Compute some additional data
    
    invalid_cycles=df['cycles'] < 0
    invalid_instructions=df['instructions'] < 0
    # CLAMP cycles and instructions
    df.loc[invalid_cycles,'cycles'] = 0
    df.loc[invalid_instructions,'instructions'] = 0
    df['IPC'] = df['instructions'] / df['cycles']
    df['IPC'] = df['IPC'].fillna(0)

    df['frequency'] =  (df['cycles'] / df['usefulTime'])
    # Normalize to seconds
    df["elapsedTime"] = df["elapsedTime"] * 1e-9

    execution_mode = detect_execution_mode(df)
    if execution_mode == ExecutionMode.HYBRID: 
        # Then we do the X
        df['totalThreads'] = (df.loc[:,'totalNumCpus'] / df.loc[:,'totalNumMpiRanks']).astype(int)
        df['ressourceLabel'] = df.loc[:,'totalNumMpiRanks'].astype(str) + "xMPI " + df.loc[:,'totalThreads'].astype(str) +"xOpenMP"
    elif execution_mode == ExecutionMode.SERIAL:
        df['ressourceLabel'] = ExecutionMode.SERIAL.value
    else:
        df['ressourceLabel'] = df.loc[:,'totalNumCpus'].astype(str) + f"x{execution_mode.value}"
    
    df['executionMode'] = execution_mode.value
    # finally sort
    df = df.sort_values(by=["elapsedTime"], ascending=False)
    return df



def load_talp_json_df(json_path: Path) -> pd.DataFrame:
    with open(json_path) as file:
        run_json = json.load(file)
        df = None
        # First check version
        if run_json[TALP_VERSION_KEY] in ["3.5","3.5a"]:
            df = __load_talp_json_3_5(run_json,json_path)
        else:
            raise ValueError(f"Unable to find parser for {run_json[TALP_VERSION_KEY]}. Please use a compatible TALP-Pages release for DLB version {run_json[TALP_VERSION_KEY]}")
        return df



def get_dfs(run_folders: List[RunFolder], as_list=True) -> Union[List[pd.DataFrame],pd.DataFrame] :
    dfs_outer =[]
    for run_folder in run_folders:
        dfs_inner = []
        for json_entry in run_folder.jsons:
            df = load_talp_json_df(json_entry)
            df['runFolder'] = str(run_folder.relative_path)
            dfs_inner.append(df)
            
        # Now combine the inner semantically clustered jsons and add to list 
        df_of_run_folder = pd.concat(dfs_inner)
        dfs_outer.append(df_of_run_folder)

    if as_list:
        return dfs_outer
    else:
        return pd.concat(dfs_outer)

def get_ressource_combinations(run_folder: RunFolder):
    df = get_dfs([run_folder],as_list=True)[0] # we only have one
    return df.sort_values(by='totalNumCpus')['ressourceLabel'].drop_duplicates().to_numpy()



import numpy as np
import plotly.graph_objects as go
from plotly.colors import sample_colorscale

from talp_pages.common import TALP_METRIC_TO_NAME_MAP,TALP_METRIC_TO_NESTING_MAP
from talp_pages.common import ExecutionMode
from talp_pages.analysis.scaling_analysis import ScalingAnalysisResult


class ScalingTableHtmlPlot:
    def __init__(self,result:ScalingAnalysisResult):
        self.result = result

    def _get_table(self):
        ressource_combinations = self.result.ressource_combinations
        df_datapoints = self.result.dataframe_list
    
        if ExecutionMode.HYBRID.value in self.result.execution_modes:
            eff_metrics=['globalEfficiency','parallelEfficiency',
                     'mpiParallelEfficiency','mpiCommunicationEfficiency','mpiLoadBalance','mpiLoadBalanceIn','mpiLoadBalanceOut',
                     'ompParallelEfficiency','ompSchedulingEfficiency','ompLoadBalance','ompSerializationEfficiency',
                    'computationScalability',
                     'instructionScaling','ipcScaling','frequencyScaling']
            abs_metrics=['IPC','frequency','elapsedTime']


        elif ExecutionMode.MPI.value in self.result.execution_modes:
            eff_metrics=['globalEfficiency','parallelEfficiency',
                     'mpiParallelEfficiency','mpiCommunicationEfficiency','mpiLoadBalance','mpiLoadBalanceIn','mpiLoadBalanceOut',
                     'computationScalability',
                     'instructionScaling','ipcScaling','frequencyScaling']
            abs_metrics=['IPC','frequency','elapsedTime']
        
        elif ExecutionMode.OPENMP.value in self.result.execution_modes:
            eff_metrics=['globalEfficiency','parallelEfficiency',
                     'ompParallelEfficiency','ompSchedulingEfficiency','ompLoadBalance','ompSerializationEfficiency',
                     'computationScalability',
                     'instructionScaling','ipcScaling','frequencyScaling']
            abs_metrics=['IPC','frequency','elapsedTime']
        
        elif ExecutionMode.SERIAL.value in self.result.execution_modes:
            eff_metrics=['globalEfficiency',
                     'computationScalability',
                     'instructionScaling','ipcScaling','frequencyScaling']
            abs_metrics=['IPC','frequency','elapsedTime']

        
        metrics = eff_metrics+abs_metrics

        header = ["Metrics"]
        header.extend([f"<b>{combination}</b>" for combination in ressource_combinations])
        aligns= ["left"]
        aligns.extend(["center" for _ in metrics ])
        values = [[f"{'-'*TALP_METRIC_TO_NESTING_MAP[metric]} {TALP_METRIC_TO_NAME_MAP[metric]}" for metric in metrics]]
        numeric_values = [datapoint[metrics].to_numpy().squeeze().round(2) for datapoint in df_datapoints]
        values.extend(numeric_values)
        
        colors = [np.full_like(metrics,'rgb(250, 250, 250)').squeeze()]
       
        colors.extend([sample_colorscale('rdylgn',np.clip(numeric_value,0,1),low=0.0,high=1.0) for numeric_value in numeric_values])
        # reset color for absolute metrics
        for col in colors[1:]:
            col[len(eff_metrics):len(metrics)]=['rgb(250, 250, 250)' for _ in abs_metrics]
        
        fig = go.Figure(data=[go.Table(
        header=dict(
            values=header,
            #line_color='white', fill_color='white',
            align=aligns,
            font=dict(color='black', size=20)
        ),
        cells=dict(
            values=values,
            line_color=colors,
            fill_color=colors,
            height=30,
            align=aligns,
            font=dict(color='black', size=16)
            ))
        ])
        return fig

    

    def get_html(self)->str:
        fig = self._get_table()
        fig.update_layout(
            autosize=True,
            height=650,
            margin=dict(l=50, r=50, b=0, t=0, pad=4),
            font=dict(
                family="Roboto Condensed, monospace",
                # size=18,
            ),
        )
        return fig.to_html(full_html=False)



#!/usr/bin/env python

import argparse

import talp_pages.cli.pages as pages
import talp_pages.cli.metadata as metadata
def main():
    parser = argparse.ArgumentParser(prog='talp', description='Command line tool of TALP-Pages.')
    subparsers = parser.add_subparsers(title='features' ,help="",dest='features')
 
    pages_parser = subparsers.add_parser('pages',help='Generate a collection of HTML pages.')
    metadata_parser = subparsers.add_parser('metadata',help='Add metdata to JSONS')
    pages.add_arguments(pages_parser)
    metadata.add_arguments(metadata_parser)
        
    args = parser.parse_args()

    if args.features == 'pages':
        pages.main(args)
    elif args.features == 'metadata':
        metadata.main(args)
    else:
        parser.print_help()

if __name__ == '__main__':
    main()



from talp_pages.common import TALP_PAGES_VERSION
import pathlib
from jinja2 import Environment, FileSystemLoader
from datetime import datetime

TALP_PAGES_TEMPLATE_PATH = pathlib.Path(__file__).parent.joinpath("templates").resolve()

class HTMLSite:
    def __init__(self,template_name: str, file_name:str) -> None:
        self.page_metadata={'page_metadata':
            {
            'timestamp_string': datetime.now().isoformat(timespec="seconds"),
            'version': TALP_PAGES_VERSION,
            'home': "../index.html",
            }
        }
        self.jinja_env = Environment(loader=FileSystemLoader(TALP_PAGES_TEMPLATE_PATH))
        self.template_name = template_name
        self.template =self.jinja_env.get_template(self.template_name)
        self.file_name = file_name

    def render_template(self,**context)->str:
        # Render the template with the provided context
        return self.template.render({**context,**self.page_metadata})
    
    def get_content(self)->str:
        raise NotImplementedError("Subclass needs to overrride this method")


    
class HTMLChildPage(HTMLSite):
    def __init__(self,template_name:str,file_name:str, icon:str) -> None:
        super().__init__(template_name,file_name)
        self.latest_change=None
        self.icon = icon
        
        




from talp_pages.html.site import HTMLChildPage
from talp_pages.analysis.time_series_analysis import TimeSeriesAnalysisResult
from typing import List
from talp_pages.plots.time_series_html_plot import TimeSeriesHtmlPlot


class TimeSeriesSite(HTMLChildPage):
    def __init__(self,result:TimeSeriesAnalysisResult,regions: List[str]) -> None:
        rc = result.ressource_configuration.replace(" ","_")
        super().__init__('time_series.jinja',file_name=f"evolution_{rc}.html",icon="bi-graph-up")
        self.latest_change = result.latest_change
        self.result= result
        self.highlited_regions = regions

    def get_content(self):
        if not self.result: 
            raise ValueError("Handed no results to TimeSeriesSites")
        plot = TimeSeriesHtmlPlot(result=self.result,regions=self.highlited_regions)
        html = self.render_template(figure=plot.get_html())
        return html


#!/usr/bin/env python

import argparse
import os
import logging
from dataclasses import dataclass
from typing import Union
from talp_pages.io.run_folders import get_run_folders
import json
from datetime import datetime

#TODO: Add timestamp
def add_arguments(parser: argparse.ArgumentParser):
    # Adding argument for JSON file
    parser.add_argument(
        "-i", "--input-path", dest="path", help="Root path to search the *.json files in", required=True
    )
    parser.add_argument("--log-level", choices=['DEBUG', 'INFO', 'WARNING', 'ERROR'],dest="log_level",help="Logger Level" ,default="WARNING")
    parser.add_argument("--git-commit", dest="git_commit",help="Git commit to put into the metadata")
    parser.add_argument("--git-branch", dest="git_branch",help="Git branch to put into the metadata")
    parser.add_argument("--git-timestamp", dest="git_timestamp",help="Timestamp (in ISO 8601 format) of git commit to put into the metadata")
    parser.add_argument("--is-merge-request", action=argparse.BooleanOptionalAction,dest="is_merge_request",help="Git branch to put into the metadata")

def _verify_input(args):
    path = None

    # Check if the JSON file exists
    if os.path.exists(args.path):
        found_a_json = False
        for _, _, filenames in os.walk(args.path):
            for _ in [f for f in filenames if f.endswith(".json")]:
                found_a_json = True

        if not found_a_json:
            logging.error(
                f"The specified path '{args.path}' doesnt contain any .json files "
            )
            raise Exception("Path empty of .json files")
    else:
        logging.error(f"The specified path '{args.path}' does not exist.")
        raise Exception("Not existing path")

    path = args.path

    return path


@dataclass
class Metadata:
    git_commit: str
    git_branch: str
    git_timestamp: datetime
    default_git_branch: str
    is_merge_request: bool

    def dict(self) -> dict:
        return {"gitCommit": self.git_commit,
                "gitBranch": self.git_branch,
                "gitTimestamp": self.git_timestamp.replace(tzinfo=None).isoformat(),
                "defaultGitBranch": self.default_git_branch,
                "isMergeRequest": self.is_merge_request}


UNKNOWN_METADATA = Metadata("unkown commit","unkown branch",None,"unkown default branch",False) 


def parse_date(date_string: str)->datetime:
    # workaround until python 3.11 where datetime support reading timeszoned strings
    if(date_string.endswith("Z")):
        date_string = date_string[:-1]
    dt = datetime.fromisoformat(date_string)
    return dt

def get_metadata(args : Union[argparse.Namespace,None] ) -> Metadata:
    metadata = UNKNOWN_METADATA
    set_from_cli=False
    
    if args:
        # first try if its specified in the args, then dont use autodetect
        if args.git_commit:
            metadata.git_commit = f"{args.git_commit}"
            set_from_cli=True
        
        if args.git_branch:
            metadata.git_branch = args.git_branch
            set_from_cli=True

        if args.is_merge_request:
            metadata.is_merge_request = args.is_merge_request
            set_from_cli=True
    
    if set_from_cli:
        logging.debug("User specified command line arguments, so we dont autodetect the metadata")
        return metadata
    # gitlab Merge request 
    if "CI_MERGE_REQUEST_ID" in os.environ:
        metadata.git_commit = os.environ.get("CI_COMMIT_SHA",UNKNOWN_METADATA.git_commit)
        metadata.git_branch = os.environ.get("CI_MERGE_REQUEST_SOURCE_BRANCH_NAME",UNKNOWN_METADATA.git_branch)
        metadata.git_timestamp = parse_date(os.environ.get("CI_COMMIT_TIMESTAMP",UNKNOWN_METADATA.git_timestamp))
        metadata.default_git_branch = os.environ.get("CI_DEFAULT_BRANCH",UNKNOWN_METADATA.default_git_branch)
        metadata.is_merge_request = True
        logging.debug("Auto detected Metadata from GITLAB Merge Request")
        return metadata
    
    # gitlab normal pipeline
    if "CI_COMMIT_SHA" in os.environ:
        metadata.git_commit = os.environ.get("CI_COMMIT_SHA",UNKNOWN_METADATA.git_commit)
        metadata.git_branch = os.environ.get("CI_COMMIT_BRANCH",UNKNOWN_METADATA.git_branch)
        metadata.git_timestamp = parse_date(os.environ.get("CI_COMMIT_TIMESTAMP",UNKNOWN_METADATA.git_timestamp))
        metadata.is_merge_request = False
        logging.debug("Auto detected Metadata from GITLAB normal pipeline")
        return metadata

    # Github Pull request
    if "GITHUB_HEAD_REF" in os.environ:
        metadata.git_commit = os.environ.get("GITHUB_SHA",UNKNOWN_METADATA.git_commit)
        metadata.git_branch = os.environ.get("GITHUB_HEAD_REF",UNKNOWN_METADATA.git_branch)
        metadata.default_git_branch = "GitHub doenst expose the default branch via ENV"
        metadata.is_merge_request = True
        logging.debug("Auto detected Metadata from Github Pull Request")
        return metadata

     # Github Normal action
    if "GITHUB_REF_NAME" in os.environ:
        metadata.git_commit = os.environ.get("GITHUB_SHA",UNKNOWN_METADATA.git_commit)
        metadata.git_branch = os.environ.get("GITHUB_REF_NAME",UNKNOWN_METADATA.git_branch)
        metadata.default_git_branch = "GitHub doenst expose the default branch via ENV"
        metadata.is_merge_request = False  
        logging.debug("Auto detected Metadata from Github normal Action")
        return metadata

    logging.error("Couldnt determine the metadata. Please use the command line interface to specify them.")
    return metadata


def add_metadata(metadata :Metadata,talp_json) -> None:
    metadata_dict =  metadata.dict()

    if "metadata" in talp_json.keys():
        for key in metadata_dict.keys():
            if key in talp_json['metadata'].keys():
                logging.error(f"metadata.{key} already present in JSON")
                raise Exception(f"metadata.{key} already present in JSON")
        
        logging.info("Adding new metadata to existing metadata")
        already_present_metadata = talp_json['metadata']
        talp_json['metadata'] = {**already_present_metadata, **metadata_dict}
    else:
        talp_json['metadata'] = metadata_dict


def main(parsed_args):
    path = _verify_input(parsed_args)
   
    log_level = getattr(logging, parsed_args.log_level.upper(), None)
    logging.basicConfig(level=log_level)

    # First detect the folder structure and append to the runs
    run_folders = get_run_folders(path)
    
    metadata = get_metadata(parsed_args)

    for run_folder in run_folders:
        for json_file_path in run_folder.jsons:
            # Open all the jsons but do in-place replacement of content
            talp_json = None
            with open(json_file_path,'r') as file:
                logging.debug(f"Start processing file {json_file_path}")
                talp_json = json.load(file)
                add_metadata(metadata,talp_json)
            with open(json_file_path,'w') as file:
                file.write(json.dumps(talp_json,indent=4))
                logging.debug(f"Done processing file {json_file_path}")


from typing import List, Tuple
import pandas as pd
import logging
from enum import Enum
from talp_pages.io.run_folders import RunFolder
from talp_pages.io.dataframe_handling import ExecutionMode
from talp_pages.io.dataframe_handling import get_dfs
from dataclasses import dataclass
from datetime import datetime
@dataclass
class ScalingAnalysisArguments:
    run_folder: RunFolder


class ScalingMode(Enum):
    WEAK = "Weak Scaling"
    STRONG = "Strong Scaling"

@dataclass
class ScalingAnalysisResult:
    region: str
    scaling_mode: ScalingMode
    scaling_table: pd.DataFrame
    execution_modes: ExecutionMode
    latest_change: Tuple[datetime,str] # Timestamp and git commit if available
    ressource_combinations: List[str]
    dataframe_list: List[pd.DataFrame]



class ScalingAnalysis:
    def __init__(self, arguments: ScalingAnalysisArguments) -> None:
        if not arguments:
            raise ValueError("No ScalingAnalysisArguments given")
        self.run_folder = arguments.run_folder
        # get df from run_folders
        self.df = get_dfs([self.run_folder], as_list=False)



    def __detect_scaling_mode(self,dfs:List[pd.DataFrame]) -> ScalingMode:
        # Assume a ordered list where the ressource with the lowest is the first
        # Use instructions to estimate the weak or strong scaling
        # Assumption: IFF Weak scaling the number of instructions/totalNumCpus should be roughly the same within a bound of +- 50%
        # Default mode. Strong
        if not dfs:
            return ScalingMode.STRONG

        base_case = dfs[0]

        if base_case['instructions'].values[0] <= 0: 
            logging.warning(f"Unable to detect ScalingMode, as no instructions are available. Run with PAPI. Assuming {ScalingMode.STRONG}")
            return ScalingMode.STRONG

        # Assume Weak scaling -> if condition violated to strong scaling
        for df in dfs:
            instruction_growth =df['instructions'].to_numpy() / base_case['instructions'].to_numpy()
            cpu_growth = df['numCpus'].to_numpy() / base_case['numCpus'].to_numpy()

            if (instruction_growth / cpu_growth) < 0.75:
                logging.debug(f"ScalingMode found to be Strong as {instruction_growth / cpu_growth} < 0.75")
                return ScalingMode.STRONG

        return ScalingMode.WEAK
                
    


    def __compute_scaling_metrics(self,region: str) -> Tuple[pd.DataFrame, pd.DataFrame,List[pd.DataFrame]]:
        # Computes the scaling metrics and returns a tuple with the first beeing just the basic_anylsis like table 
        # and the second one being a ordered list of the dataframes making up the BA table.   
      
        clean_df = self.df.loc[self.df['regionName'] == region]
        if clean_df.empty:
            raise ValueError("Region not available")

        
        ressource_combinations = clean_df.sort_values(by='totalNumCpus')['ressourceLabel'].drop_duplicates().to_numpy()
        df_datapoints = []
        for combination in  ressource_combinations:
            datapoint = clean_df.loc[clean_df['ressourceLabel'] == combination].sort_values(by=["timestamp"], ascending=False).head(1)
            df_datapoints.append(datapoint)
        if not df_datapoints:
            logging.error(f"Not able to compute the scaling metrics for dataframe: {clean_df}")
            #return (pd.DataFrame(),[])
        
        scaling_mode = self.__detect_scaling_mode(df_datapoints)

        clean_df.loc[:,"scalingMode"] = scaling_mode.value

        reference_case = df_datapoints[0]

        if scaling_mode==ScalingMode.STRONG:
            #Assume instructions stay constant 
            clean_df['instructionScaling'] = (reference_case['instructions'] / clean_df.loc[:,'instructions'])
        if scaling_mode == ScalingMode.WEAK:
            clean_df['instructionScaling'] = (reference_case['instructions']/reference_case['numCpus']) * (1/(clean_df['instructions']/clean_df['numCpus']))

        clean_df.loc[:,'ipcScaling']= clean_df.loc[:,'IPC'] / reference_case['IPC']
        clean_df.loc[:,'frequencyScaling'] =clean_df.loc[:,'frequency'] / reference_case['frequency']
        
        # NO Papi cases
        clean_df['instructionScaling'] = clean_df['instructionScaling'].fillna(0) 
        clean_df['ipcScaling'] = clean_df['ipcScaling'].fillna(0) 
        clean_df['frequencyScaling'] = clean_df['frequencyScaling'].fillna(0)

        clean_df.loc[:,'computationScalability']= clean_df['instructionScaling'] * clean_df['ipcScaling'] *  clean_df['frequencyScaling']
        clean_df.loc[:,'globalEfficiency'] = clean_df.loc[:,'parallelEfficiency'] * clean_df.loc[:,'computationScalability']
        clean_df.loc[:,'speedup'] = reference_case['elapsedTime'] / clean_df['elapsedTime']
        clean_df.loc[:,'globalEfficiencyBySpeedup'] = clean_df.loc[:,'speedup'] / (clean_df.loc[:,'numCpus'] / reference_case['numCpus'])
        
        df_datapoints = []
        for combination in  ressource_combinations:
            datapoint = clean_df[clean_df['ressourceLabel'] ==combination].sort_values(by=["timestamp"], ascending=False).head(1)
            df_datapoints.append(datapoint)
        return (pd.DataFrame(),df_datapoints,scaling_mode,ressource_combinations)
            
    def get_results(self,regions: List[str]) -> List[ScalingAnalysisResult]:
        # Returns a result for every region
        results = []
        for region in regions:
            scaling_table, list_of_dfs,scaling_mode,ressource_combinations = self.__compute_scaling_metrics(region)
            execution_modes = [df['executionMode'].to_numpy() for df in list_of_dfs]
            latest_date = None
            latest_change = None
            for df in list_of_dfs:
                date = pd.to_datetime(df['timestamp'].to_numpy().squeeze())
                git_commit = str(df['gitCommitShort'].to_numpy().squeeze())
                
                try: 
                    if date > latest_date:
                        latest_date = date
                        latest_change = (date,git_commit)
                except TypeError:
                    latest_change = (date,git_commit)
                
            results.append(ScalingAnalysisResult(region=region,scaling_mode=scaling_mode,scaling_table=scaling_table,execution_modes=execution_modes,latest_change=latest_change,ressource_combinations=ressource_combinations,dataframe_list=list_of_dfs))

        return results


from talp_pages.html.site import HTMLChildPage
from talp_pages.analysis.scaling_analysis import ScalingAnalysisResult
from typing import List
from talp_pages.plots.scaling_table_html_plot import ScalingTableHtmlPlot
import logging

from operator import itemgetter

class ScalingResultSite(HTMLChildPage):
    def __init__(self, results: List[ScalingAnalysisResult]) -> None:
        super().__init__(template_name='scaling.jinja',file_name="scaling.html",icon="bi-graph-up")
        self.results = results
        self.latest_change = max([result.latest_change for result in self.results], key=itemgetter(1)) # Get newest change
        
    def get_content(self):
        if not self.results: 
            raise ValueError("Handed no results to ScalingResultSite")

        regions = [result.region for result in self.results]
        scaling_mode = self.results[0].scaling_mode # just take the first
        
        configurations = {}
        regions_to_render=[]
        
        for region_name,result in zip(regions,self.results):
            configurations[region_name] =[]
            plot = ScalingTableHtmlPlot(result)
            try:
                figure = plot.get_html()
            except ValueError as e: 
                logging.warning(f"Trying to get region with name  {region_name} for scaling analysis but failed. Skipping that region... {e}")
            
            for datapoint in result.dataframe_list:               
                configurations[region_name].append(
                    {'label': datapoint['ressourceLabel'].to_numpy().squeeze(),
                     'json_file':  datapoint['jsonFile'].to_numpy().squeeze(),
                     'timestamp': datapoint['timestamp'].to_numpy().squeeze(),
                     'metadata': datapoint['metadata'].to_numpy().squeeze()
                    }
                )
                scaling_mode = datapoint['scalingMode'].to_numpy().squeeze()
        
            if figure:
                regions_to_render.append({'name': region_name,'figure': figure})
        
        html = self.render_template(regions=regions_to_render,configurations=configurations,scaling_mode=scaling_mode)
        return html
    



from talp_pages.io.run_folders import RunFolder
from typing import List
from pathlib import Path
from dataclasses import dataclass

from talp_pages.analysis.scaling_analysis import ScalingAnalysis, ScalingAnalysisArguments
from talp_pages.analysis.time_series_analysis import TimeSeriesAnalysis, TimeSeriesAnalysisArguments
from talp_pages.analysis.badge_analysis import BadgeAnalysis
from talp_pages.io.dataframe_handling import get_ressource_combinations
from talp_pages.html.scaling_result_site import ScalingResultSite
from talp_pages.html.time_series_result_site import TimeSeriesSite
from talp_pages.html.ci_report_index_page import CIReportIndexPage
from talp_pages.svg.badge import Badge
from talp_pages.io.output import write_badge,write_site
from operator import itemgetter

@dataclass
class CIReportArgs:
    run_folders: List[RunFolder]
    selected_regions: List[str]
    region_for_badge: str
    with_badge: bool



class CIReport():
    def __init__(self, args:CIReportArgs) -> None:
        self.run_folders=args.run_folders
        self.selected_regions=args.selected_regions
        self.region_for_badge=args.region_for_badge
        self.with_badge=args.with_badge

    def write_to_folder(self,output_path:Path):
        sites={}
        # for each run folder get the sites
        for run_folder in self.run_folders:
            # first get scaling result site one per region
            scaling_results = ScalingAnalysis(ScalingAnalysisArguments(run_folder=run_folder)).get_results(self.selected_regions)
            scaling_result_site = ScalingResultSite(scaling_results)
            # one entry per ressource combination
            ressource_configurations = get_ressource_combinations(run_folder)
            time_series_results= TimeSeriesAnalysis(TimeSeriesAnalysisArguments(run_folder=run_folder)).get_results(ressource_configurations)
        
            
            time_series_sites = []
            for rc,result in zip(ressource_configurations,time_series_results):
                time_series_sites.append(TimeSeriesSite(result,self.selected_regions))
            
            badges=[]
            if self.with_badge:
                bagde_results = BadgeAnalysis(run_folder=run_folder,region_for_badge=self.region_for_badge).get_results(ressource_configurations=ressource_configurations)
                for rc,result in zip(ressource_configurations,bagde_results):
                    badges.append(Badge(result))
            
            
            sites[run_folder.relative_path] = {
                'scaling_result_site':scaling_result_site,
                'ressource_configurations':ressource_configurations,
                'badges': badges,
                'region_for_badge': self.region_for_badge,
                'time_series_sites': time_series_sites,
                'relative_path':run_folder.relative_path,
                'latest_change':max([site.latest_change for site in [scaling_result_site,*time_series_sites]], key=itemgetter(1)) # Get newest across subsites
            }
            # Write out the things
            subfolder = output_path.joinpath(run_folder.relative_path)
            for site in [scaling_result_site,*time_series_sites]:
                write_site(subfolder,site)
            for badge in badges:
                write_badge(subfolder,badge)

         

        index_page = CIReportIndexPage(subsites=sites)
        write_site(output_path,index_page)

    
            




#!/usr/bin/env python

import argparse
import os
import logging
from talp_pages.io.run_folders import get_run_folders

from talp_pages.common import TALP_IMPLICIT_REGION_NAME
from talp_pages.reports.ci_report import CIReport, CIReportArgs
from pathlib import Path
def add_arguments(parser: argparse.ArgumentParser):
    parser.add_argument(
        "-i", "--input-path", dest="path", help="Root path to search the *.json files in",required=True
    )
    parser.add_argument(
        "-o",
        "--output-path",
        dest="output",
        help="relative path for the output",
        required=True,
    )
    parser.add_argument('--regions', nargs='+', help='Names of regions of that will be highlited seperated by spaces',dest="regions")
    parser.add_argument('--region-for-badge', help=f"Name of region that will be used to generate the performance badges (Default: {TALP_IMPLICIT_REGION_NAME})",dest="region_for_badge",default=TALP_IMPLICIT_REGION_NAME)
    parser.add_argument('--badge', action=argparse.BooleanOptionalAction,default=True, help="Generate Parallel effiency badge .svg (Requires internet)")
    parser.add_argument("--log-level", choices=['DEBUG', 'INFO', 'WARNING', 'ERROR'],dest="log_level",help="Logger Level" ,default="ERROR")
    

def _verify_input(args):
    path = None

    # Check if the JSON file exists
    if os.path.exists(args.path):
        found_a_json = False
        for _, _, filenames in os.walk(args.path):
            for _ in [f for f in filenames if f.endswith(".json")]:
                found_a_json = True

        if not found_a_json:
            logging.error(
                f"The specified path '{args.path}' doesnt contain any .json files "
            )
            raise Exception("Path empty of .json files")
    else:
        logging.error(f"The specified path '{args.path}' does not exist.")
        raise Exception("Not existing path")

    # Check if output is empty
    if os.path.exists(args.output):
        if len(os.listdir(args.output)) != 0:
            logging.error(f"The specified output folder '{args.output}' is not empty")
            raise Exception("Non Empty output path")

    path = args.path
    output = args.output
    regions = [TALP_IMPLICIT_REGION_NAME]
    if args.regions:
        regions.extend(args.regions)

    region_for_badge = args.region_for_badge

    return path, output,regions,region_for_badge


def main(parsed_args):
    input_path, output_path, selected_regions,region_for_badge = _verify_input(parsed_args)

    log_level = getattr(logging, parsed_args.log_level.upper(), None)
    logging.basicConfig(level=log_level)
    
    run_folders = get_run_folders(input_path)
    report = CIReport(CIReportArgs(run_folders,selected_regions,region_for_badge,parsed_args.badge))
    report.write_to_folder(Path(output_path))

            
        


    

    

